﻿#nullable disable

using System.Data;
using System.Data.SqlClient;
using System.Reflection;

namespace AdoNetWrapper.Validation.Common;

/// <summary>
/// Database context using ADO.NET 
/// for SQL Server Databases
/// </summary>
public partial class SqlServerDatabaseContext : DatabaseContext {

  public SqlServerDatabaseContext(string connectString) : base(connectString) { }

  protected override void Init() {
    base.Init();

    ParameterPrefix = "@";
  }

  public override SqlConnection CreateConnection(string connectString) {
    if (IsInTransaction) {
      if (ConnectionObject == null) {
        ConnectionObject = new SqlConnection(connectString);
      }
      return (SqlConnection)ConnectionObject;
    }
    else {
      return new SqlConnection(connectString);
    }
  }

  public override SqlCommand CreateCommand(IDbConnection cnn, string sql) {
    CommandObject = new SqlCommand(sql, (SqlConnection)cnn);
    CommandObject.CommandType = CommandType.Text;

    if (IsInTransaction) {
      CommandObject.Transaction = TransactionObject;
    }

    return (SqlCommand)CommandObject;
  }

  public override SqlParameter CreateParameter(string paramName, object value) {
    if (!paramName.StartsWith(ParameterPrefix)) {
      paramName = ParameterPrefix + paramName;
    }
    return new SqlParameter(paramName, value);
  }

  public override SqlParameter CreateParameter() {
    return new SqlParameter();
  }

  public override SqlParameter GetParameter(string paramName) {
    if (!paramName.StartsWith(ParameterPrefix)) {
      paramName = ParameterPrefix + paramName;
    }

    return ((SqlCommand)CommandObject).Parameters[paramName];
  }

  public override SqlDataReader CreateDataReader(IDbCommand cmd, CommandBehavior cmdBehavior = CommandBehavior.CloseConnection) {
    // Open Connection
    cmd.Connection.Open();
    // Create DataReader
    DataReaderObject = cmd.ExecuteReader(cmdBehavior);

    return (SqlDataReader)DataReaderObject;
  }

  public override void GetLastAutoIncrement<TEntity>(IDbCommand cmd, TEntity entity, PropertyInfo prop) {
    cmd.CommandText = "SELECT CONVERT(INT, @@IDENTITY);";
    cmd.CommandType = CommandType.Text;
    int? value = (int?)cmd.ExecuteScalar();
    if (value.HasValue) {
      prop.SetValue(entity, value, null);
    }
  }
}
